///////////////////////////////////////////////////////////////////////////////////////////
// MAT 259 Assignment 4: Bird Migration Patterns Visualization
// Drawing Code (General) 
// by Nataly Moreno
//
// Links to download libraries:
//     ControlP5: http://www.sojamo.de/libraries/controlP5/#installation
//
///////////////////////////////////////////////////////////////////////////////////////////

final int DISPLAY_TIME = 1500;
final float MAX_ALPHA = 130.0;

float margin = 0.03;

//Fonts
PFont font1 = createFont("Arial", 22, true);     
PFont font2 = createFont("Helvetica", 27, true); 
PFont font3 = createFont("Arial Bold", 50, true);
PFont font4 = createFont("Arial", 12, true);
PFont font5 = createFont("Arial Bold", 24, true);
PFont font6 = createFont("Arial Bold", 10, true);
//PFont font7 = createFont("Arial", 10, true);



color modifiable = color(0, 0, 0, 0);
color blueColor     = color(8, 139, 225, 0);
color darkGrayColor = color(51, 50, 50, 202);
color darkBlueColor = color(50, 121, 162, 241);

String[] monthLabel = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",
                        "Aug", "Sep", "Oct", "Nov", "Dec" };

ArrayList<String> birdLinks = new ArrayList<String>();

//For the Framerate
int rate = 30;

//For Fading Animation
float alphi = 7.5 * MAX_ALPHA * (float)rate / DISPLAY_TIME;

int time = 0;
float alphaVal = MAX_ALPHA;
int lastTime;

//For Animation
String selectedBird = "";
float circleIncrease = 1.0;

int currentMonth = 0;
int current_week = 0;

//TimeLine Animation
float barLength = 396.0; //Has to be divisible by 4
float startTime = 0;
float step = (float)(barLength - ((float)barLength / 12.0)) / (float)(DISPLAY_TIME * 48.0);
float oldWidth = 0;

float cellWidth  = 3;
float cellHeight = 15;

void draw() 
{
  background(240);
  //background(244, 243, 223, 0);    
  noFill();
  strokeWeight(1);
  stroke(modifiable);
     
  //Title
  fill(darkGrayColor);
  textAlign(LEFT, TOP);
  textFont(font3);
  text("Annual Bird Migration", width * margin, height * margin);
   
  //Draw ControlP5 GUI
  //drawGUI();
    
  //Draw Bird Bar Graph
  drawBarGraph();
  
  //Draw Every Bird Photo With Name and Color Bar
  drawBirdKey();
}

//////////////////////////////////////////////////////////////////////////////
///////////////////////////// Draw Bird Bar Graph ////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void drawBarGraph()
{
  colorMode(RGB);
  stroke(255, 255, 0);
  strokeWeight(1);
  fill(255, 0, 0);
  
   for(int i = 0, currSt = 0; i < 48 * 6; i += 6, currSt++) //Weekly Data (Columns)
   {     
     for(int j = 0; j < 50; j++) //States (Rows)
     {
       for(int k = 0; k < 6; k++)
       {
         int stDataAt;
         float fbird;
         
         //stroke(255, 255, 0); //yellow
         stroke(50);
         strokeWeight(0.7);
         //Bird_Data tempAviData = birdyData.get("American Black Duck");

         Bird_Data tbird = birdyData.get(BirdSpecies.get(k));
         //.bird_states.get(states.get(j));
         //BirdSpecies.indexOf(searchTerm), i
         //float fbird = tbird.bird_states.get(states.get(j));
         stDataAt = tbird.bird_states.indexOf(states[j]);
         if(stDataAt == -1)
           fbird = 0;
         else
           fbird = tbird.weekly_bird_data.get(stDataAt).get(currSt); //j = birdstate i = week
         
         cellColor(k, fbird);
         rect((width * margin * 3) + (cellWidth * (i + k)), 
              (height * margin * 4) + j * cellHeight, 
               cellWidth, cellHeight);
         
         stroke(0, 255, 0);
         
         if(k == 5 && j == 0)
         {
           line((width * margin * 3) + (cellWidth * (i + k + 1)),
                (height * margin * 4) + 50 * cellHeight,
                (width * margin * 3) + (cellWidth * (i + k + 1)),
                (height * margin * 4) + 50 * cellHeight + 10);
                
           fill(darkGrayColor);
           textAlign(CENTER, CENTER);
           textFont(font6);
           text(currSt + 1, (width * margin * 3) + (cellWidth * (i + k + 1)),
                (height * margin * 4) + 50 * cellHeight + 20);
         }
       }
     }
   }
   
   fill(darkGrayColor);
   textAlign(CENTER, CENTER);
   textFont(font5);
   text("Week of the Year", 
        (width * margin * 3) + (cellWidth * ((48 * 6) + 6 + 1)) / 2 ,
        (height * margin * 4) + 50 * cellHeight + 50);
   
   //Write State Names
   fill(darkGrayColor);
   textAlign(LEFT, TOP);
   textFont(font6);
   
   for(int i = 0; i < 50; i++)
   {
     text(states[i], width * margin, height * margin * 4 + (i * cellHeight + 4));
   }
   
   /*println("MIN: " + minBirdCount);
   println("MAX: " + maxBirdCount);
   println("rawmin: " + rawMin);
   println("rawmax: " + rawMax);
   */
   
   noLoop();
}

void drawBirdKey()
{
  for(int i = 0; i < BirdSpecies.size(); i++)
  {
    fill(darkGrayColor);
    textAlign(LEFT, TOP);
    textFont(font6);
    text(BirdSpecies.get(i), 
         (width * margin * 0.4) + (cellWidth * 48 * 7) + 25,
          height * margin * 4 + (i * 130) );
    birdLinks.add(GoogleImageSetup(BirdSpecies.get(i)));
    drawImageFromGoogle(BirdSpecies.get(i), birdLinks.get(i), 80, 
                       (width * margin * 0.4) + (cellWidth * 48 * 7) + 25,
                        height * margin * 4 + (i * 130) + 30);
  }
}

void drawColorBar(String searchTerm, float barx, float bary)
{
  int barDiv = 100;
  int barLength = 100;
  int barHeight = 14;
  
  //Fill Color Bar
  for(int i = 0; i < barDiv; i++)
  {
    colorMode(RGB);
    noStroke();
    colorBarFill(BirdSpecies.indexOf(searchTerm), i);    
    rect(barx + (i * (barLength / barDiv)), bary, 
         barLength / barDiv, barHeight);
  }

  
  //Draw Color Bar Outline
  noFill();
  colorMode(RGB);
  stroke(0);
  strokeWeight(1);
  rect(barx, bary, barLength, barHeight);
}








/*int findIndexOf(String toFind)
{
  int index = -1;
  for (int i = 0; i < states.length; i++) 
  {
    if (states[i].equals(toFind)) 
    {
        index = i;
        break;
    }
  }
  
  return index;
}
*/
